/* 
    Copyright (C) 2004  Mika Raento - Renaud Petit

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


    email: mraento@cs.helsinki.fi - petit@cs.helsinki.fi 
*/
#include "user_notifier.h"
#include "timeout.h"
#include "Contextclientsession.h"
#include "symbian_auto_ptr.h"
#include "contextcommon.h"
#include "eikenv.h"
#include "aknquerydialog.h"
#include "aknmessagequerydialog.h"
#include <context_log.rsg>
#include "browser_interface.h"
#include <e32std.h>
#include <CFLDRingingTonePlayer.h>
#include <viewcli.h> //CVwsSessionWrapper
#include <vwsdef.h>

_LIT(KClassName, "CUserNotifierImpl");


class CUserNotifierImpl : public CUserNotifier, public CCheckedActive, public MContextBase, public MTimeOut {
public:
	~CUserNotifierImpl();
private:
	CUserNotifierImpl(MApp_context& Context, CFLDRingingTonePlayer * aRingp, Clog_profile * aPl);
	void ConstructL();

	void CheckedRunL();
	TInt CheckedRunError(TInt aError);
	void DoCancel();

	void expired();

	void Start();
	void Restart();

	void RequestMessageNotification();
	void CancelRequest();

	void NotifyUser(TDesC& contact, TDesC& subject, TDesC& message);

	RContextClientSession iSession;
	bool iSessionOpen;
	CTimeOut* iWait;

	enum TState
	{
      		EIdle,
        	EWaitingForMessage		
	};
	TState current_state;

	TPtr iC;
	TPtr iS;
	TPtr iM;

	HBufC * iContact;
	HBufC * iSubject;
	HBufC * iMessage;

	CFLDRingingTonePlayer * ringp;
	Clog_profile * pl;

	//XML_Parser		iParser;

	const static TInt KDefaultBufferSize;

	friend class CUserNotifier;
};

const TInt CUserNotifierImpl::KDefaultBufferSize=512;

CUserNotifier* CUserNotifier::NewL(MApp_context& Context, CFLDRingingTonePlayer * aRingp, Clog_profile * aPl)
{
	CUserNotifierImpl * self = new (ELeave) CUserNotifierImpl(Context, aRingp, aPl);
    	CleanupStack::PushL(self);
    	self->ConstructL();
	CleanupStack::Pop();

    	return self;
}

CUserNotifierImpl::CUserNotifierImpl(MApp_context& Context, CFLDRingingTonePlayer * aRingp, Clog_profile * aPl) : CCheckedActive(EPriorityNormal, KClassName), MContextBase(Context), iC(0,0), iS(0,0), iM(0,0), ringp(aRingp), pl(aPl)
{
    	CActiveScheduler::Add(this);
}

void CUserNotifierImpl::ConstructL()
{
	iWait=CTimeOut::NewL(*this);
	current_state = EIdle;

	/*iParser = XML_ParserCreate(NULL);

	XML_ParserReset(iParser, NULL);
	XML_SetUserData(iParser,this);
	XML_SetElementHandler(iParser, CUserNotifierImpl::startElement, CUserNotifierImpl::endElement);
	XML_SetCharacterDataHandler(iParser, CUserNotifierImpl::charData);
*/
	iContact = HBufC::NewL(KDefaultBufferSize/4);
	iSubject = HBufC::NewL(KDefaultBufferSize/4);
	iMessage = HBufC::NewL(KDefaultBufferSize);

	Start();
}

void CUserNotifierImpl::Start()
{
	if (iSession.ConnectToContextServer() == KErrNone)
	{
		iSessionOpen=true;
		RequestMessageNotification();
	} else {
		Restart();
	}
}

void CUserNotifierImpl::Restart()
{
	Cancel();
	iWait->Cancel();
	if (iSessionOpen) {
		iSession.Close();
	}
	iSessionOpen=false;
	//RDebug::Print(_L("restarting"));
	iWait->Wait(20);
}


void CUserNotifierImpl::expired()
{
	Start();
}

CUserNotifierImpl::~CUserNotifierImpl()
{
	Cancel();
	if (iSessionOpen) iSession.Close();
	delete iWait;

	delete iContact;
	delete iSubject;
	delete iMessage;
	//XML_ParserFree(iParser);
}

void CUserNotifierImpl::RequestMessageNotification()
{
	if(!IsActive())
	{
		current_state = EWaitingForMessage;

		iC.Set(iContact->Des());
		iS.Set(iSubject->Des());
		iM.Set(iMessage->Des());

		iSession.MsgRequestMessageNotification(iC, iS, iM, iStatus);
		SetActive();
	}		
}

void CUserNotifierImpl::CancelRequest()
{
    	Cancel() ;
}

void CUserNotifierImpl::DoCancel()
{
	iSession.Cancel();
	current_state = EIdle;
}

void CUserNotifierImpl::CheckedRunL()
{
	if (iStatus == ERequestCompleted)
	{
		switch (current_state)
		{
			case EWaitingForMessage:
			{
//				RDebug::Print(iC);
//				RDebug::Print(iS);
//				RDebug::Print(iM);

				NotifyUser(iC,iS,iM);
				
				RequestMessageNotification();
				break;
			}
			default:
				Restart();
				// ASSERT(0); //Unexpected error
				break;
		}
	}
	else if (iStatus == EBufferTooSmall)
	{
		iContact = iContact->ReAllocL(iC.MaxLength() *2);
		iC.Set(iContact->Des());

		iSubject = iSubject->ReAllocL(iS.MaxLength() *2);
		iS.Set(iSubject->Des());

		iMessage = iMessage->ReAllocL(iM.MaxLength() *2);
		iM.Set(iMessage->Des());

		RequestMessageNotification();
	}
	else 
	{
		Restart();
	}
}

TInt CUserNotifierImpl::CheckedRunError(TInt aError)
{
	return aError;
}

void CUserNotifierImpl::NotifyUser(TDesC& contact, TDesC& subject, TDesC& message)
{
	/* EXPECTED FORMAT
	<alert>
		<description>What to show to the user</description>
		<url>...</url>
	</alert>
	*/
	
	

	//FIXME:Quick and dirty, should use expat parser!

	if ( message.Find(_L("<alert>"))== KErrNotFound || message.Find(_L("</alert>"))== KErrNotFound )
	{
		return;	
	}

	//unhide
	RWsSession& wsSession=CEikonEnv::Static()->WsSession();
	TApaTask task(wsSession);
	task.SetWgId(CEikonEnv::Static()->RootWin().Identifier());
	task.BringToForeground();
	((MCoeForegroundObserver*)ringp)->HandleGainingForeground();

	int description_b = message.Find(_L("<description>"));
	int description_e = message.Find(_L("</description>"));
	int url_b = message.Find(_L("<url>"));
	int url_e = message.Find(_L("</url>"));

	TPtrC desc = message.Mid(description_b + 13, description_e - 13 - description_b); 
	TPtrC url  = message.Mid(url_b + 5, url_e - 5 - url_b); 

	//-----------------------------
	
	int profile = pl->get_current_profile();
	// cannot play tone if we want to play a ringtone
	//CAknQueryDialog::TTone tone=CAknQueryDialog::EWarningTone;
	CAknQueryDialog::TTone tone=CAknQueryDialog::ENoTone; 
	ringp->SetVibra(ETrue);
	ringp->SetRingingType(2); // ring once
	switch (profile) {
	case 2:
		// meeting
		((MFLDFileProcessor*)ringp)->ProcessFileL(_L("c:\\system\\apps\\context_log\\silent.rng"));
		break;
	case 1:
		// silent
		break;
	default:
		// general, outdoor
		((MFLDFileProcessor*)ringp)->ProcessFileL(_L("c:\\system\\apps\\context_log\\tone.rng"));
		break;
	}

	CAknMessageQueryDialog* dlg = CAknMessageQueryDialog::NewL(desc, tone);
		 
	CleanupStack::PushL(dlg);
	dlg->PrepareLC(R_MESSAGE_QUERY);
	dlg->QueryHeading()->SetTextL(_L("Notification"));
	CleanupStack::Pop();
	
	if ( dlg->RunLD() )
	{
		#ifdef __WINS2__
		CEikonEnv::Static()->BusyMsgL(url, 2000);
		#endif

		#ifndef __WINS2__
		
		#  ifndef __S60V2__
		auto_ptr<CDorisBrowserInterface> ido(CDorisBrowserInterface::NewL());
		ido->AppendL(CDorisBrowserInterface::EOpenURL_STRING, url);
		ido->ExecuteL();
		#  else
		HBufC8* addr8=HBufC8::NewLC(url.Length());
		TPtr8 addrp=addr8->Des();
		CC()->ConvertFromUnicode(addrp, url);
		TUid KUidOperaBrowserUid = {0x101F4DED};
		TUid KUidOperaRenderViewUid = {0};
		TVwsViewId viewId(KUidOperaBrowserUid, KUidOperaRenderViewUid);
		CVwsSessionWrapper* vws;
		vws=CVwsSessionWrapper::NewLC();
		vws->ActivateView(viewId, TUid::Uid(0), *addr8);
		CleanupStack::PopAndDestroy(2);
		#  endif

		#endif
	}
	((MCoeForegroundObserver*)ringp)->HandleLosingForeground();
	((MFLDFileProcessor*)ringp)->Cancel();
	
}